import urllib
from chargebee import util, http_request
from chargebee.main import ChargeBee
from chargebee import compat
import json


def send_list_request(method, url, params=None, env=None, headers=None):
    serialized = {}

    if params is None:
        params = {}

    for k, v in list(params.items()):
        if isinstance(v, list):
            v = json.dumps(v)
        serialized.update({k: v})
    return send(method, url, serialized, env, headers)


def send(method, url, params=None, env=None, headers=None):
    if params is None:
        params = {}

    env = env or ChargeBee.default_env

    ser_params = util.serialize(params)

    response = http_request.request(method, url, env, ser_params, headers)

    from chargebee.result import Result
    from chargebee.list_result import ListResult
    if 'list' in response:
        return ListResult(response['list'], response.get('next_offset', None))
    return Result(response)


def uri_path(*paths):
    url = ""
    for path in paths:
        if path is None or len(str(path).strip()) < 1:
            raise Exception("Id is None or empty")
        if compat.py_major_v >= 3:
            url = url + "/" + urllib.parse.quote(str(path).strip())
        else:
            url = url + "/" + urllib.quote(str(util.get_val(path)))
    return url



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class Usage(Model):

    fields = ["id", "usage_date", "subscription_id", "item_price_id", "invoice_id", "line_item_id", \
    "quantity", "source", "note", "resource_version", "updated_at", "created_at"]


    @staticmethod
    def create(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"usages"), params, env, headers)

    @staticmethod
    def retrieve(id, params, env=None, headers=None):
        return request.send('get', request.uri_path("subscriptions",id,"usages"), params, env, headers)

    @staticmethod
    def delete(id, params, env=None, headers=None):
        return request.send('post', request.uri_path("subscriptions",id,"delete_usage"), params, env, headers)

    @staticmethod
    def list(params, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("usages"), params, env, headers)

    @staticmethod
    def pdf(params, env=None, headers=None):
        return request.send('post', request.uri_path("usages","pdf"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class TaxWithheld(Model):

    fields = ["id", "reference_number", "description", "date", "currency_code", "amount"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class QuotedCharge(Model):
    class Charge(Model):
      fields = ["amount", "amount_in_decimal", "description", "service_period_in_days", "avalara_sale_type", "avalara_transaction_type", "avalara_service_type"]
      pass
    class Addon(Model):
      fields = ["id", "quantity", "unit_price", "quantity_in_decimal", "unit_price_in_decimal", "service_period"]
      pass
    class InvoiceItem(Model):
      fields = ["item_price_id", "quantity", "quantity_in_decimal", "unit_price", "unit_price_in_decimal", "service_period_days"]
      pass
    class ItemTier(Model):
      fields = ["item_price_id", "starting_unit", "ending_unit", "price", "starting_unit_in_decimal", "ending_unit_in_decimal", "price_in_decimal"]
      pass
    class Coupon(Model):
      fields = ["coupon_id"]
      pass

    fields = ["charges", "addons", "invoice_items", "item_tiers", "coupons"]




import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class ItemPrice(Model):
    class Tier(Model):
      fields = ["starting_unit", "ending_unit", "price", "starting_unit_in_decimal", "ending_unit_in_decimal", "price_in_decimal"]
      pass
    class TaxDetail(Model):
      fields = ["tax_profile_id", "avalara_sale_type", "avalara_transaction_type", "avalara_service_type", "avalara_tax_code", "hsn_code", "taxjar_product_code"]
      pass
    class AccountingDetail(Model):
      fields = ["sku", "accounting_code", "accounting_category1", "accounting_category2", "accounting_category3", "accounting_category4"]
      pass

    fields = ["id", "name", "item_family_id", "item_id", "description", "status", "external_name", \
    "pricing_model", "price", "price_in_decimal", "period", "currency_code", "period_unit", "trial_period", \
    "trial_period_unit", "trial_end_action", "shipping_period", "shipping_period_unit", "billing_cycles", \
    "free_quantity", "free_quantity_in_decimal", "resource_version", "updated_at", "created_at", \
    "archived_at", "invoice_notes", "tiers", "is_taxable", "tax_detail", "accounting_detail", "metadata", \
    "item_type", "archivable", "parent_item_id", "show_description_in_invoices", "show_description_in_quotes"]


    @staticmethod
    def create(params, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices"), params, env, headers)

    @staticmethod
    def retrieve(id, env=None, headers=None):
        return request.send('get', request.uri_path("item_prices",id), None, env, headers)

    @staticmethod
    def update(id, params=None, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices",id), params, env, headers)

    @staticmethod
    def list(params=None, env=None, headers=None):
        return request.send_list_request('get', request.uri_path("item_prices"), params, env, headers)

    @staticmethod
    def delete(id, env=None, headers=None):
        return request.send('post', request.uri_path("item_prices",id,"delete"), None, env, headers)

    @staticmethod
    def find_applicable_items(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("item_prices",id,"applicable_items"), params, env, headers)

    @staticmethod
    def find_applicable_item_prices(id, params=None, env=None, headers=None):
        return request.send('get', request.uri_path("item_prices",id,"applicable_item_prices"), params, env, headers)



import json
from chargebee.model import Model
from chargebee import request
from chargebee import APIError

class QuotedSubscription(Model):
    class Addon(Model):
      fields = ["id", "quantity", "unit_price", "amount", "trial_end", "remaining_billing_cycles", "quantity_in_decimal", "unit_price_in_decimal", "amount_in_decimal"]
      pass
    class EventBasedAddon(Model):
      fields = ["id", "quantity", "unit_price", "service_period_in_days", "on_event", "charge_once", "quantity_in_decimal", "unit_price_in_decimal"]
      pass
    class Coupon(Model):
      fields = ["coupon_id"]
      pass
    class SubscriptionItem(Model):
      fields = ["item_price_id", "item_type", "quantity", "quantity_in_decimal", "metered_quantity", "last_calculated_at", "unit_price", "unit_price_in_decimal", "amount", "amount_in_decimal", "free_quantity", "free_quantity_in_decimal", "trial_end", "billing_cycles", "service_period_days", "charge_on_event", "charge_once", "charge_on_option"]
      pass
    class ItemTier(Model):
      fields = ["item_price_id", "starting_unit", "ending_unit", "price", "starting_unit_in_decimal", "ending_unit_in_decimal", "price_in_decimal"]
      pass
    class QuotedContractTerm(Model):
      fields = ["contract_start", "contract_end", "billing_cycle", "action_at_term_end", "total_contract_value", "cancellation_cutoff_period"]
      pass

    fields = ["id", "plan_id", "plan_quantity", "plan_unit_price", "setup_fee", "billing_period", \
    "billing_period_unit", "start_date", "trial_end", "remaining_billing_cycles", "po_number", "auto_collection", \
    "plan_quantity_in_decimal", "plan_unit_price_in_decimal", "changes_scheduled_at", "change_option", \
    "contract_term_billing_cycle_on_renewal", "addons", "event_based_addons", "coupons", "subscription_items", \
    "item_tiers", "quoted_contract_term"]




import unittest

from chargebee import util


class UtilTest(unittest.TestCase):

    def test_serialize(self):
        before = {
            'id': 'sub_KyVq7DNSNM7CSD',
            'plan_id': 'free',
            'addons': (
                {
                    'id': 'monitor',
                    'quantity': 2,
                },
                {
                    'id': 'ssl',
                }
            ),
            'card': {
                'first_name': 'Rajaraman',
                'last_name': 'Santhanam',
                'number': '4111111111111111',
                'expiry_month': '1',
                'expiry_year': '2024',
                'cvv': '007',
            }
        }

        after = {
            'id': 'sub_KyVq7DNSNM7CSD',
            'plan_id': 'free',
            'addons[id][0]': 'monitor',
            'addons[quantity][0]': 2,
            'addons[id][1]': 'ssl',
            'card[first_name]': 'Rajaraman',
            'card[last_name]': 'Santhanam',
            'card[number]': '4111111111111111',
            'card[expiry_month]': '1',
            'card[expiry_year]': '2024',
            'card[cvv]': '007',
        }

        self.assertEqual(after, util.serialize(before))
